import pytest

from pypayment import LavaPayment, Payment, AuthorizationError, NotAuthorized, PaymentStatus
from test import lava_token, lava_wallet


@pytest.mark.skip(reason="Lava API is not working")
def test_payment_creation_without_authorization():
    with pytest.raises(NotAuthorized):
        LavaPayment(1)


@pytest.mark.skip(reason="Lava API is not working")
def test_authorization_with_invalid_key():
    with pytest.raises(AuthorizationError):
        LavaPayment.authorize("invalid token", lava_wallet)


@pytest.mark.skip(reason="Lava API is not working")
def test_authorization():
    LavaPayment.authorize(lava_token, lava_wallet)
    assert LavaPayment.authorized


@pytest.mark.skip(reason="Lava API is not working")
def test_payment_creation():
    payment: Payment = LavaPayment(1)
    assert payment is not None


@pytest.mark.skip(reason="Lava API is not working")
def test_url_getting():
    payment: Payment = LavaPayment(1)
    assert "https://acquiring.lava.kz/invoice/" in payment.url or \
           "https://acquiring.lava.ru/invoice/" in payment.url


@pytest.mark.skip(reason="Lava API is not working")
def test_status_getting():
    payment: Payment = LavaPayment(1)
    payment.update()
    assert payment.status == PaymentStatus.WAITING


@pytest.mark.skip(reason="Lava API is not working")
def test_income_getting():
    payment: Payment = LavaPayment(1)
    payment.update()
    assert payment.income == 1



from .exceptions import NotAuthorized, PaymentCreationError, PaymentGettingError, AuthorizationError, PaymentNotFound
from .enums.commission import ChargeCommission
from .enums.status import PaymentStatus
from .payment import Payment
from .providers.qiwi import QiwiPayment, QiwiPaymentType
from .providers.yoomoney import YooMoneyPayment, YooMoneyPaymentType
from .providers.payok import PayOkPayment, PayOkPaymentType, PayOkCurrency
from .providers.lava import LavaPayment
from .providers.betatransfer import BetaTransferPayment, BetaTransferPaymentType, BetaTransferCurrency, \
    BetaTransferGateway, BetaTransferLocale



from environs import Env

env = Env()
env.read_env()

qiwi_secret_key = env.str("qiwi_secret_key")
yoomoney_access_token = env.str("yoomoney_access_token")
payok_api_key = env.str("payok_api_key")
payok_api_id = env.str("payok_api_id")
payok_shop_id = env.str("payok_shop_id")
payok_shop_secret_key = env.str("payok_shop_secret_key")
lava_token = env.str("lava_token")
lava_wallet = env.str("lava_wallet")



from abc import ABC, abstractmethod
from typing import Optional, Tuple
from uuid import uuid4

from pypayment import NotAuthorized, PaymentStatus, PaymentNotFound


class Payment(ABC):
    """Payment interface than allows to create and check invoices."""

    authorized = False
    """Is payment class authorized."""

    def __init__(self, amount: float, description: str = "", id: Optional[str] = None) -> None:
        self._check_authorization()

        self.id: str = id if id is not None else str(uuid4())
        """Unique Payment ID (default: generated with uuid4)."""

        self.amount: float = round(amount, 2)
        """Amount to be invoiced. May not include commission."""

        self.description: str = description if description else str(self.id)
        """Payment comment."""

        self.status: PaymentStatus = PaymentStatus.WAITING
        """Payment status. Use update() to update it."""

        self.income: Optional[float] = None
        """Payment income. Use update() to update it."""

        self._validate_params()

        self.url: str = self._create_url()
        """Payment URL."""

    @classmethod
    @abstractmethod
    def get_status_and_income(cls, payment_id: str) -> Tuple[Optional[PaymentStatus], float]:
        """Returns payment status and income.

        :param payment_id: Payment ID.
        :raises PaymentNotFound: Payment not found.
        :return: Payment status and income.
        """

    def update(self) -> None:
        try:
            status, income = self.__class__.get_status_and_income(self.id)
        except PaymentNotFound:
            return

        if status:
            self.status = status
        self.income = income

    @abstractmethod
    def _create_url(self) -> str:
        """Creates payment URL."""

    def _check_authorization(self) -> None:
        """Raises NotAuthorized if class was not authorized."""
        if not self.authorized:
            raise NotAuthorized(f"You need to authorize first: {self.__class__.__name__}.authorize()")

    def _validate_params(self) -> None:
        """Validates payment parameters."""
        pass
